/*

Copyright (C) 2009 Jaroslav Barton <djaara@djaara.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <QLabel>
#include <QHBoxLayout>
#include <QPixmap>

#include <KStandardDirs>

#include "EnrollStatus.h"

/**
 * EnrollStatus - Widget with big fingeprint image and small status icon.
 * @param fingerprintFileName path to fingerprint image
 * @param enrollOkFileName path to enroll ok status icon
 * @param enrollErrFileName path to enroll error status icon
 * @param parent parent widget
 */
EnrollStatus::EnrollStatus(const QString& fingerprintFileName, const QString& enrollOkFileName, const QString &enrollErrFileName, QWidget *parent)
    : QWidget(parent), 
    m_enrollState(false) 
{
	initComponents();
	loadImages(fingerprintFileName, enrollOkFileName, enrollErrFileName);
	setConstraints();
}

/**
 * EnrollStatus - Widget with big fingerprint image and small status icon.
 * This constructor uses predefined images. If you want define your own images,
 * use @see EnrollStatus(const QString& fingerprintFileName, const QString& enrollOkFileName, const QString &enrollErrFileName, QWidget *parent)
 * @param parent parent widget
 */
EnrollStatus::EnrollStatus(QWidget *parent) 
    : QWidget(parent), 
      m_enrollState(false) 
{
	initComponents();
	loadImages(KStandardDirs::locate("data", "kfingermanager/pics/fingerprint.png"),
		   KStandardDirs::locate("data", "kfingermanager/pics/fp-ok.png"),
		   KStandardDirs::locate("data", "kfingermanager/pics/fp-error.png"));
	setConstraints();
}

/**
 * setConstraints - set fixed width for whole widget and for status label.
 */
void EnrollStatus::setConstraints() {
	m_statusLabel->setPixmap(*m_okPixmap);
	m_statusLabel->setFixedWidth(m_statusLabel->minimumSizeHint().width());
	setFixedSize(minimumSizeHint());
	clearStatus();
}

/**
 * loadImages - load images needed by widget.
 * @param fingerprintFileName path to fingerprint image
 * @param enrollOkFileName path to enroll ok status icon
 * @param enrollErrFileName path to enroll error status icon
 */
void EnrollStatus::loadImages(const QString& fingerprintFileName, const QString& enrollOkFileName, const QString& enrollErrFileName) {
	m_fingerprintLabel->setPixmap(QPixmap(fingerprintFileName));
	m_okPixmap = new QPixmap(enrollOkFileName);
	m_errPixmap = new QPixmap(enrollErrFileName);
}

/**
 * initComponents - initialize all components of widget.
 */
void EnrollStatus::initComponents() {
	QHBoxLayout *hb = new QHBoxLayout(this);
	m_fingerprintLabel = new QLabel(this);
	m_statusLabel = new QLabel(this);

	m_fingerprintLabel->setAlignment(Qt::AlignBottom);
	m_statusLabel->setAlignment(Qt::AlignBottom);

	hb->addWidget(m_fingerprintLabel);
	hb->addWidget(m_statusLabel);
	
	setLayout(hb);
}

/**
 * setChecked - set status icon.
 * @param ok True means use enroll ok status icon, false means use enroll error status icon
 */
void EnrollStatus::setChecked(bool ok) {
	m_enrollState = ok;
	if (ok) {
		m_statusLabel->setPixmap(*m_okPixmap);
	} else {
		m_statusLabel->setPixmap(*m_errPixmap);
	}
}

/**
 * clearStatus - cleans status icon. Status icon will be cleaned only if status
 * icon is error status icon.
 */
void EnrollStatus::clearStatus() {
	if (!m_enrollState) {
		m_statusLabel->clear();
	}
}

/**
 * ~EnrollStatus - destructor.
 */
EnrollStatus::~EnrollStatus() {
	delete m_statusLabel;
	delete m_fingerprintLabel;
	delete m_okPixmap;
	delete m_errPixmap;
}
