/*

Copyright (C) 2009 Jaroslav Barton <djaara@djaara.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <QFont>
#include <QLabel>
#include <QCheckBox>
#include <QDebug>
#include <QTimer>
#include <QVBoxLayout>
#include <QHBoxLayout>

#include <KLocale>

#include "FMEnroll.h"
#include "FingerNames.h"

/**
 * FMEnroll constructor
 * @param device device selected
 * @param loginName users login
 * @param finger selected finger, use "Right index finger" as default
 * @param parent parent widget
 */
FMEnroll::FMEnroll(DeviceModel device, const QString &loginName, int finger, QWidget *parent) 
    : KDialog(parent), 
      m_statusLabel(NULL),
      m_actualStatusLabel(NULL),
      m_stages(NULL),
      m_animLabel(NULL),
      m_mainPanel(NULL),
      m_finger(finger), 
	  m_loginName(loginName),
      m_device(device), 
      m_dbc(NULL),
      m_enrollStages(0),
      m_actualStage(0),
      m_scanType("swipe"),
      m_timer(NULL)
{
	// Modal dialog, only Cancel button, precedent by separator
	setModal(true);
	showButtonSeparator(true);
	setButtons(KDialog::Cancel);
	
	// D-Bus connection
	m_dbc = FMDbusConn::getConn();
	getIface();
	
	// Init and traslate UI components
	initComponents(m_enrollStages);
	retranslate();

	setMainWidget(m_mainPanel);
	
	// Layout
	QVBoxLayout *mainLayout = new QVBoxLayout();

	QHBoxLayout *statusLayout = new QHBoxLayout();
	statusLayout->addWidget(m_statusLabel);
	statusLayout->addWidget(m_actualStatusLabel);
	statusLayout->addStretch();
	mainLayout->addLayout(statusLayout);
	
	QHBoxLayout *stagesLayout = new QHBoxLayout();
	stagesLayout->addStretch();
	for (int i = 0; i < m_enrollStages; i++) {
		stagesLayout->addWidget(m_stages[i]);
	}
	stagesLayout->addStretch();
	stagesLayout->addWidget(m_animLabel);
	
	mainLayout->addLayout(stagesLayout);
	m_mainPanel->setLayout(mainLayout);
	
	setMaximumSize(minimumSizeHint());
	setMinimumSize(minimumSizeHint());

	if (!m_dbc->isDeviceClaimed()) {
		m_actualStatusLabel->setText(i18n("Error while claiming device."));
	} else {
		m_dbc->enrollStart(fingerNames[m_finger].iface);
		m_animLabel->start();
	}
}

/**
 * getIface - create new D-Bus interface object, reads scan type and number of enroll stages.
 */
void FMEnroll::getIface() {
	m_dbc->claimDevice(m_device.devicePath, m_loginName);
	if (m_dbc->isDeviceClaimed()) {
		m_enrollStages = m_dbc->getEnrollStages();
		m_scanType = m_dbc->getScanType();
		connect(m_dbc, SIGNAL(EnrollStatus(QString,bool)), this, SLOT(enrollStatusSlot(QString,bool)));
	} else {
		m_scanType = "";
	}
}

/**
 * initCimponents - init all widgets
 * @param enrollStages number of enroll stages
 */
void FMEnroll::initComponents(int enrollStages) {
	m_mainPanel = new QWidget();

	m_statusLabel = new QLabel(this);
	QFont font = m_statusLabel->font();
	font.setWeight(QFont::Bold);
	m_statusLabel->setFont(font);
	
    m_actualStatusLabel = new QLabel(this);

	m_timer = new QTimer();
	m_timer->setSingleShot(true);
	connect(m_timer, SIGNAL(timeout()), this, SLOT(timeoutSlot()));

	m_animLabel = new SensorAnimLabel(m_scanType);
	m_animLabel->stop();

    if (enrollStages > 0) {
        m_stages = (EnrollStatus **)malloc(sizeof(EnrollStatus *) * enrollStages);
        for (int i = 0; i < enrollStages; i++) {
            m_stages[i] = new EnrollStatus(this);
        }
    }
}

/**
 * retranslate - update all UI strings
 */
void FMEnroll::retranslate() {
	// Set Window Title
	setWindowTitle(i18n("Enrolling finger..."));
	
	// What's this
	setWhatsThis(i18n("Enroll finger window"));
	
	// Set status label
	m_statusLabel->setText(i18n("Status")+":");
	
	// Set stages tooltips
	for (int i = 0; i < m_enrollStages; i++) {
		QString msg = i18n("Stage %1").arg(i + 1);
		m_stages[i]->setToolTip(msg);
		m_stages[i]->setWhatsThis(i18n("Current enrollment progess..."));
	}
	
	// Status message
	timeoutSlot();
}

/**
 * setStatusMessage - set suitable status message for actual enroll status
 * @param result actual enroll status
 */
/*
# enroll-completed:  The enrollment successfully completed, Device.EnrollStop should now be called.
# enroll-failed: The enrollment failed, Device.EnrollStop should now be called.
# enroll-stage-passed: One stage of the enrollment passed, the enrollment is still ongoing.
# enroll-retry-scan: The user should retry scanning their finger, the enrollment is still ongoing.
# enroll-swipe-too-short: The user's swipe was too short. The user should retry scanning their finger, the enrollment is still ongoing.
# enroll-finger-not-centered: The user's finger was not centered on the reader. The user should retry scanning their finger, the enrollment is still ongoing.
# enroll-remove-and-retry: The user should remove their finger from the reader and retry scanning their finger, the enrollment is still ongoing.
# enroll-disconnected: The device was disconnected during the enrollment, no other actions should be taken, and you shouldn't use the device any more.
# enroll-unknown-error: An
*/
void FMEnroll::setStatusMessage(const QString& result) {
	int timeout = 2500;
	bool checked = false;

	if (result == "enroll-stage-passed") {
		checked = true;
		m_actualStatusLabel->setText(i18n("Stage passed"));
		timeout = 1250;
	} else if (result == "enroll-swipe-too-short") {
		m_actualStatusLabel->setText(i18n("Swipe too short"));
	} else if (result == "enroll-finger-not-centered") {
		m_actualStatusLabel->setText(i18n("Finger not centered"));
	} else if (result == "enroll-remove-and-retry") {
		m_actualStatusLabel->setText(i18n("Remove and retry"));
	} else if (result == "enroll-retry-scan") {
		m_actualStatusLabel->setText(i18n("Retry scan"));
	}
	
	m_stages[m_actualStage]->setChecked(checked);
	m_animLabel->stop();
	
	m_timer->start(timeout);

	if (checked) {
		m_actualStage++;
	}
}

/**
 * enrollStatus - Slot for asynchronous enroll signals
 * @param result actual enroll status
 * @param done true if enroll process is completed, otherwise false
 */
void FMEnroll::enrollStatusSlot(const QString &result, bool done) {
	qDebug() << "enrol result:" << result << ", done: " << done;

	if (done == true) {
		if (result == "enroll-completed") {
			m_actualStatusLabel->setText(i18n("Enroll completed"));
			m_stages[m_actualStage]->setChecked(true);
			setButtons(KDialog::Ok);
		} else if (result == "enroll-disconnected") {
			m_actualStatusLabel->setText(i18n("Device disconected. Fingerprint is not enrolled."));
		} else if (result == "enroll-failed") {
			m_actualStatusLabel->setText(i18n("Enroll failed."));
		} else if (result == "enroll-unknown-error") {
			m_actualStatusLabel->setText(i18n("Unknown error."));
		}
		
        m_dbc->releaseDevice();
		m_animLabel->stop();
	} else {
        setStatusMessage(result);
    }
}

/**
 * timeout - handle timeout and set proper message
 */
void FMEnroll::timeoutSlot() {
	// Create actual status message
	QString msg;
	if (m_scanType == "swipe") {
		msg = QString(i18n("Swipe your \"%1\" on \"%2\""));
	} else {
		msg = QString(i18n("Place your \"%1\" on \"%2\""));
	}
	msg = msg.arg(i18n(fingerNames[m_finger].name), m_device.deviceName);
	m_actualStatusLabel->setText(msg);
	m_stages[m_actualStage]->clearStatus();
	m_animLabel->start();
}

/**
 * FMEnroll destructor
 */
FMEnroll::~FMEnroll() {
	disconnect(this, SLOT(enrollStatusSlot(QString, bool)));
	
	if (m_stages) {
		for (int i = 0; i < m_enrollStages; i++) {
			if (m_stages[i]) delete m_stages[i];
		}
		free(m_stages);
	}

	delete m_animLabel;
	delete m_mainPanel;
	
    if (m_dbc) {
		m_dbc->releaseDevice();
		m_dbc->releaseConn();
	}
}
